     TITLE "Controller for OCXO Discipline by GPS - Brooks Shera, W5OJM, September, 2008"

;Copyright  1997-2008, Brooks Shera.  All rights reserved.  This program
;may be used only for non-commercial purposes, and carries no warranty
;of any kind, including any implied warranty of fitness for any particular
;purpose. 
 
; Redistribution and use in source and binary forms, with or without
; modification, are permitted for non-commercial purposes only 
; provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright
;    notice, this list of conditions and the following disclaimer.
;
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
;
; This software is provided by the author and contributors "as is" and
; any express or implied warranties, including, but not limited to, the
; implied warranties of merchantability and fitness for a particular 
; purpose are disclaimed. In no event shall the author or contributors 
; be liable for any direct, indirect, incidental, special, exemplary, 
; or consequential damages (including, but not limited to, procurement
; of substitute goods or services; loss of use, data, or profits; or 
; business interruption) however caused and on any theory of liability, 
; whether in contract, strict liability, or tort (including negligence 
; or otherwise) arising in any way out of the use of this software, 
; even if advised of the possibility of such damage. 

;*********************************************************************
;      Define Processor - comment out all but the device selected 
;*********************************************************************

;    LIST n=58, p=PIC16C73
;    errorlevel  1
;    include P16C73.INC
;    __CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF

;*********************************************************************

;    LIST n=58, p=PIC16F73
;    errorlevel  1
;    include P16F73.INC
;    __CONFIG _HS_OSC & _BODEN_OFF & _CP_OFF & _PWRTE_ON & _WDT_OFF

;*********************************************************************

;    LIST n=58, p=PIC16F873A
;    errorlevel  1
;    include P16F873A.INC
;    __CONFIG _HS_OSC & _CP_OFF & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF

;*********************************************************************

    LIST n=58, p=PIC16F876A
    errorlevel  1
    include P16F876A.INC
    __CONFIG _HS_OSC & _CP_OFF & _DEBUG_OFF & _WRT_OFF  & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF

;*********************************************************************

;Note: LVP_OFF (low voltage programming) is required for normal operation of pin RB3
;Note: Watch Dog Timer(WDT) must be off since it will nominally reset CPU in 18 msec.

;*********************************************************************
;                      Controller Description
;*********************************************************************
;Can select either PIC internal counter or external counters as phase detector
;by changing the routine called by int_srv.  The former eliminates several chips.
;Note - INT_CTR added to simplify selecting the internal counter configuration
;IIR filter parameters:Ts  = 30 sec F2 = 64, F1 and Kcpu determined by PORTC lo bits
;PORTC 3 lo bits=0 selects setup mode, =1 type 1 filter (error signal direct to dac)
;
;This version has alpha=1/32, only used for N >= 4 and sw 5 open.
;
;Note:*** check that computed goto's (...addwf PCL,1...) do NOT cross 0xFF prog. ctr.
;   bounderies since only the lo 8 bits of PCL are changed - no carry occurs,
;   see AN 556 for a more robust solution ***
;
;I/O pins in use:
;   PORTA
;     0     analog output of software PWM for 1/sec phase_watcher 
;   1,2,3   freq low (glitch) LED, GPS heart beat, freq high (unlock) LED
;    4,5    prescaler & counter input, output pin to toggle prescaler   
;   PORTB
;     0     ext interrupt 1pps in
;     1     VCO polarity input (1 means inc V => inc F)
;     2     Alpha filter or DAC hold switch: Open(hi)=>use alpha/DAC hold
;     3     74HC166 shift/load control out
;     4     counter reset line out
;     5     DAC data out
;     6     DAC clock out
;     7     DAC latch out
;   PORTC
;   0,1,2   user inputs
;    3,4    SPI clock & data in
;     5     allocated to SPI data out
;     6     USART asynch xmt output
;     7     USART asynch rcv input
;
;Status bits mean:
;   bits 
;   0-7     N value as always
;   +10     glitch detected 
;   +20     alpha filter actually used |(alpha-iir)| < 50
;           which implies diff < 4.3 mv or about 1 nsec/30sec
;   +50     alpha filter enabled (sw 5 open, was open loop)
;			or open loop if Alpha is not defined 
;   +100    "unlock" LED on 
;
;DIP switch     1   2   3          4         5      6   7   8
;   port:      RC0 RC1 RC2        RB1       RB2     nc  nc  nc
;   use:     {setup,type1,iir}   VCO+/-  alpha filt
;
;*********************************************************************
;						Revision History
;*********************************************************************

;6/28/98 - setpoint changed to 800 from 1024 for better margin with 10 MHZ VCXOs
;9/27/98 - add ASCII echo code to wait loop to test USART rcv
;3/99    - add alpha post filter (conditional assembly)
;4/1/99  - init alpha filter when changing N values.  print values on a single line
;5/26/99 - add phase_watcher PWM analog output (concept & orig code by Bob Larkin)
;5/30/99 - change limiter to avoid sign change for N=1 when (data-setpoint) > 1024
;4/27/01 - add ASCII input command interperter
;5/4/01  - add change phase setpoint via ASCII input (concept & code by Bob Leichner)
;5/6/01  - add alpha filter enable based on N > 3 and |alpha-IIR| < 50 (~4 mv)
;11/15/02- delete bcf Hc,7 below mulC1 for neg numbers - causes errors on overflow
;1/26/03 - add ASCII input of DIP switches & device ID character(code by Bob Leichner)
;7/6/03 -  serial bootloader (Bob Leichner)
;11/30/03 - periodic clear of serial receive port to avoid hardware hangup bug (Bob Leichner)
;11/16/03 - wide phase capture range vsn - 10 MHz, 1/640 divider = 15,625 Hz input
;1/20/04 - reduce SPI interface speed for slow shift register  & remove deglitch for test
;2/6/04  - combine 1.355a(wide phase) & 1.34a(std) vsn so PIC can drive either type VCXO
;2/7/04  - "wide mode" selected by RA1 held weakly Hi via 1.2K from top of LO LED to +5v
;10/27/04 - add ASCII "DIP switches" and bootloader (see above) to standard version
;9/7/05  - add ASCII input to control EFC output
;10/12/05 - minor changes to ASCII input routines & comments, fixed minor bug
;2/14/08 - reduce quantization jitter:change setpoint - shoot for integ each measurement
;9/15/08 - add Auto N to increment N if phase sits near setpoint - to simplify /tvb testing
;9/23/08 - add write/read eeprom code to save/restore Nmax value and enable AutoN
;03/16/13 - Brooks Shera passes on and will be missed. 

;********************************************************************
;Modifications of the Brooks Shera code by Richard McCorkle    

;03/27/13 - Complete final Brooks Shera code review with
;           corrections made to fix the issues discovered. 
;03/31/13 - Complete initial public code re-ordered for  
;           better readability with all fixes
;04/01/13 - Corrected so if N > Nmax AutoN is disabled. Added 
;           readctr accumulator to emulate 166 for phiwatch. 
;           Added bipolar limiter and BP_LIMIT conditional
;           to allow setpoints greater than 1023 to be used.
;04/07/13 - Per BL request added conditional disabling the LOAD# 
;           bootloader command, add ID storage and restore from 
;           EEPROM, add EEPROM locations table
 
;*********************************************************************
;                     conditional assembly flags
;*********************************************************************

#define     ALPHA_FILTER          ;alpha filter available (open loop not)
#define     ASCII_INPUT           ;enable ASCII input command interperter
#define     EEPROM                ;enables EEprom code
;#define     BP_LIMIT              ;bipolar limiter for setpoints > 1023
;#define     INT_CTR               ;enable to use internal counter
;#define     WIDE_PHASE            ;low incoming VCXO freq for wide phase range
;#define     PHI_WATCH_PRINT       ;enable this line to get 1/sec printing
;#define ALPHA_FILTER_DEBUG_PRINT  ;alpha debug print

;*********************************************************************
;                        ASCII status flags
;*********************************************************************

#define JamEFC	EFCopts,0       ;set DAC from IIR (0) or from ASCII value (1)
#define S_hold 	EFCopts,2       ;run (0) or hold (1) the existing DAC setting (open loop)
#define	dGlOff	EFCopts,3       ;de-glitch enable (0) or disable (1) flag

;*********************************************************************
;      Macro Definitions -  used in eeprom read/write routines
;*********************************************************************

BANK0   MACRO
        bcf STATUS,RP0          ;clear bank select bits
        bcf STATUS,RP1
        bcf STATUS,IRP          ;clear indirect addressing bit
        ENDM
		
BANK1   MACRO
        bsf STATUS,RP0	
        bcf STATUS,RP1
        bcf STATUS,IRP          ;clear indirect addressing bit
        ENDM
		
BANK2   MACRO
        bcf STATUS,RP0	
        bsf STATUS,RP1
        bsf STATUS,IRP          ;set bit for indirect addressing
        ENDM
		
BANK3   MACRO
        bsf STATUS,RP0	
        bsf STATUS,RP1
        bsf STATUS,IRP          ;set bit for indirect addressing
        ENDM

DISABLE_IRQ MACRO               ;*** disable interrupts ***
        local stop_int
stop_int 
        bcf INTCON,GIE          ;disable global interrupt
        btfsc INTCON,GIE        ;check if disabled 
        goto stop_int           ;nope, try again
        ENDM

ENABLE_IRQ MACRO                ;*** enable interrupts ***
        bsf INTCON,GIE          ;enable global interrupt
        ENDM

;*********************************************************************
;                     define storage locations
;*********************************************************************

W_TEMP      equ 0x20    ;temp storage for w reg in int.
STAT_TEMP   equ 0x21    ;ditto for status reg
CHAR        equ 0x22    ;the next char to be xmitted

    CBLOCK  0x23        
        Lbyte,Hbyte ;storage used by read166 & binary-to-BCD routines
        R2, R1, R0  ;must stay in sequence R2, R1, R0
        count, wtmp
        Ldata,Hdata ;storage for data from counter
        tick        ;count 1 pps interrupts
        tsecs       ;seconds to count - set according to usr
        toc         ;storage for minute counter for IIR entry
        minutes     ;minutes to count - set according to usr
        Ha, Ma, La, Sa, SSa, Hb, Mb, Lb, Sb, SSb, Hc, Mc, Lc, Sc, SSc ;math regs
        Hfi, Lfi    ;storage for filter input (phase error)
        Hfil, Lfil  ;storage for previous filter input
        Hfol, Mfol, Lfol, Sfol, SSfol ;storage for previous filter output
        Haol, Maol, Laol;storage for previous alpha filter output
        Haoc, Maoc, Laoc;storage for current alpha filter output
        Hpdat,Lpdat ;storage for previous phase data input
        Hsp, Lsp    ;storage for phase set point
        Hdac, Ldac  ;storage for DAC current setting
        cLoBits     ;used by iir to set Kcpu - 3 lowest bits of PORTC
        pLoBits     ;used by adjust filter parameters - previous 3 lowest bits
        glcctr      ;used by dGlitch to limit multiple entries
      	temp        ;temporary storage
        temp2       ;more temp storage
        Hic,Lic     ;readctr accumulator 

;following registers used by phi_watch & software PWM

        Hphip       ;previous phi value (16 bits)
        Lphip       ;
        phi         ;one sec phase count difference for PWM (8 bits is enough)
        phic        ;(PWM - phi) for PWM
        pwm_on      ;counts for PWM to be logic 1
        pwm_off     ;counts for PWM to be logic 0   

;ASCII value and AutoN registers

        dipbits     ;RAM alternative to physical DIP switch. May be set using serial port
        diptemp     ;hold serial port version of dipbits until loaded in critical section
        devID       ;single character to add to end of serial output string
        Hefc        ;storage ascii input of efc set/offset 
        Lefc
        EFCopts     ;ascii efc control options
        Nctr        ;counter for number of consecutive updates with phase near setpoint
        Nmax        ;sets max value for AutoN and turns it off if 0 (value stored in EEProm)
        EE_Byte	    ;EE_WRITE_BYTE routine gets the data to be written to eeprom here 

;storage for ASCII command processor (don't touch in interrupt routines)
 
        cmd0,cmd1,cmd2 ;temp storage
        gbtemp      ;used by getByte
        gbTick      ;seconds time-out counter used by getByte
        h2btemp     ;used by hex2bin    

;when we're out of registers in bank0 can combine 6 regs below into 1 byte 

        FrstPss     ;first pass flag to init interrupt code
        FrstAlf     ;first pass thru alpha filter
        UsedAlf     ;alpha filter used for currrent output
        negFlag     ;used by multiply routine to indicate neg. value
        sclFlag     ;used by scale routine for return from mul1
        Wide        ;indicates wide phase mode selected
    ENDC

; Note: present RAM variables occupy 0x20 thru 0x75. Going above 0x7F (top of bank 0) 
; will require bank switching or changing to a 18Fxx2 series pic to get more RAM.

;*********************************************************************
;                      EEPROM Locations In Use
;*********************************************************************

;Address    Variable
;  10         Nmax    
;  11         devID

;*********************************************************************
;                 Interrupt vector and start of code
;*********************************************************************
;set interrupt vector and start of code

    org 0       	    	; initialize code
		nop					; required for the ICD 
		clrf	STATUS		; ensure we are at page 0	
		clrf    PCLATH     	; ensure bank bits are cleared
		goto	start	    ; jump to start at end of code
    org 4       	    	; interrupt routine
		goto    int_srv
 
;*********************************************************************
;                   Routines using computed goto's
;*********************************************************************
;Note: Routines using computed goto's (addwf PCL,1) are placed first 
;to insure they do NOT cross 0xFF prog. ctr. bounderies since only 
;the lo 8 bits of PCL are changed - no carry occurs
;*** see AN 556 for a more robust solution ***

;divide by F1 

divByF1 movf    cLoBits,W   ;setting of user bits selects F1
        addwf   PCL,1       ;index into table - RshBl_ returns to caller
        goto    RshBL4      ;should not get here - not using iir filter
        goto    RshBL4      ;F1=2^10 (already shifted 2^6 (64) - not iir
        goto    RshBL5      ;2^11 for N=2
        goto    RshBL6      ;2^12 for N=3
        goto    RshBL7      ;2^13 for N=4
        goto    RshBL8      ;2^14 for N=5
        goto    RshBL9      ;2^15 for N=6
        goto    RshBL10     ;2^16 for N=7

;IIR output gain multiplier

mul_IIR	movf    cLoBits,W   ;setting of users bits selects gain (Kcpu)
        andlw   0x07        ;mask to discard hi bits  debug *****
        addwf   PCL,1       ;index into table - mulC_ goes to c2dac
        goto    mulC11      ;should not get here with cLoBits=0 
        goto    mulC11      ;multiply by 2^11
        goto    mulC10      ;or 2^10 for N=2
        goto    mulC9       ;N=3
        goto    mulC8       ;N=4
        goto    mulC7       ;N=5
        goto    mulC6       ;N=6
        goto    mulC5       ;N=7

;user-selected initialize routines (selected by 3 lo bits (value 0-7) in PORTC

setFilt movf    dipbits,W   ;read user inputs (either physical or ASCII input)
        andlw   0x07        ;mask to discard hi bits
        movwf   cLoBits     ;save for mulC_ jump table indexing, etc
        addwf   PCL,1       ;index into table to initialize sample time
        goto    usr0        ;NOTE: all the sample time stuff is historical
        goto    usr1
        goto    usr2
        goto    usr3
        goto    usr4
        goto    usr5
        goto    usr6
        goto    usr7

;*********************************************************************
;                        Background Routine
;*********************************************************************
;background code runs the phi_watch PWM analog output 
;and ASCII input routines between interrupts
    
bkgnd   movf    pwm_on,F    ;set status bit Z to 1 if pwm_on==0
        btfsc   STATUS,Z    ;was pwm_on 0?
        goto    pwm0        ;yes - continue sending 0
        decfsz  pwm_on      ;no, it was 1
        goto    more1       ;continue sending 1
        bcf 	PORTA,0     ;switch port from 1 to 0
        nop         	    ;these nops equalize the time to 13
        call    nop4        ;instruction cycles, regardless of state.
        goto    bkgnd
pwm0    decfsz  pwm_off
        goto    more0       ;continue at 0 level
        bsf 	PORTA,0     ;end of zero period, set PWM to 1
        movf    phi,W
        movwf   pwm_on
        movf    phic,W
        movwf   pwm_off
        goto    bkgnd

    ifdef ASCII_INPUT

more0   nop                 ;burn same number of cycles as original more0
        nop
        btfsc   PIR1,RCIF   ;have we received a char? Branch IF NOT (normal case)
        goto    readCmd		
        goto    bkgnd       ;just loop

more1   call    nop4        ;burn same number of cycles as original more1.
        btfss   PIR1,RCIF   ;have we received a char? Branch if so...
        goto    bkgnd       ;same number of cycles if we get here (normal case)
        goto    readCmd		

    else                    ;Original delays - replaced by ASCII routines when used

more0   call    nop4        ;Check for input on both 0 and 1 PWM case (bug)
        goto    bkgnd

more1   nop                 ;continue at 1 level, again 13 instr cycles
        call    nop4
        goto    bkgnd       ;loop again

    endif                   ;ASCII_INPUT

;*********************************************************************
;                     interrupt service routine
;*********************************************************************

int_srv movwf   W_TEMP      ;"push" instructions (see AN611)
        swapf   STATUS,W
        bcf     STATUS,RP0
        bcf     STATUS,RP1  ;make sure we're on page 0 where the data storage lives
        movwf   STAT_TEMP
        bcf     INTCON,INTF ;clear interrupt, else int again immediately!
        bcf     PORTA,0     ;set PWM out to zero during foregnd to reduce glitches
        bcf     PORTA,2     ;toggle heartbeat on/off on alternate gps pulses
        btfss   tick,0      ;test low bit of tick counter
        bsf     PORTA,2     ;turn LED on if tick is even
        call    phiwtch     ;read phase once/sec and set PWM for monitoring

    ifdef ASCII_INPUT
        movf    gbTick,f    ;decrement ASCII time-out ctr if not zero
        btfss   STATUS,Z    
        decf    gbTick,f
    endif                   ;ASCII_INPUT

    ifdef PHI_WATCH_PRINT   ;want alot of ASCII output?
        call    Data2B      ;save current phase for the later filter routines
        movf    phi,W       ;put 1/sec phase diff into print locations
        movwf   Lbyte
        clrf    Hbyte
        call    Bin2BCD     ;print - convert to BCD,
        call    sendnum     ;then ASCII and transmit,
        call    prtCRNL     ;end with CR LF
        call    B2BCD       ;restore current phase to "work" registers
    endif                   ;PHI_WATCH_PRINT
 
        decfsz  tick        ;read/clear phase counter, filter, set DAC every TCOUNT sec
        goto    pop         ;otherwise just clean up & return from interrupt

;Update time - begin the heavy lifting

        movf    tsecs,W     ;reinitialize tick down counter
        movwf   tick

    ifdef INT_CTR
        clrf    Hic         ;clear readctr accumulator 
        clrf    Lic
    endif                   ;INT_CTR

;copy current dip switch settings to working register

        movf    diptemp,W   ;copy serial values to dipbits?
        movwf   dipbits
        btfss   dipbits,7   ;if bit 7 set use bits set by serial command
        call	copydip	    ;if not, get physical dip settings 
        call    setFilt     ;check setting of user inputs on each interrupt
        call    clr166      ;clear the phase counter, etc.
        call    Bin2BCD     ;convert data to BCD
        call    sendnum     ;convert to ascii & transmit - BEFORE dGlitch or limiter

    ifdef WIDE_PHASE
        btfss   Wide,0      ;test flag and avoid dGlitch if set
    endif                   ;WIDE_PHASE

        call    dGlitch     ;ignore large phase changes (only in iir modes 4 to 7)
        call    limiter     ;limit filter input - limit value depends on phase width

;compute and output OCXO control voltage with user-selected filter

        btfss   dipbits,5   ;usr input for VCO polarity - 1 means V^ => F^
        goto    revpol      ;pin 1 clear means V^ => F decreases
        call    Data2A      ;get current phase reading to A
        call    SP2B        ;get desired phase set point to B
        goto    getdiff
revpol  call    Data2B      ;current phase reading (rev. pol.for HP5328A)
        call    SP2A        ;get desired phase set point (see above)
getdiff call    Sub24       ;B - A -> B reg
        call    B2FIBCD     ;copy B to filter input store & ascii conv
;       call    print       ;print phase difference from setpoint
        movf    cLoBits,W   ;test lo bits of PORTC - 0 for setup mode 
        btfsc   STATUS,Z    ;if set at zero...
        goto    setupMd     ;branch for setup mode
        call    locktst     ;if phase near setpoint everything under control
        movf    cLoBits,W   ;get PORTC bit again
        andlw   0x06        ;test for bit 1 only set - type 1 filter
        btfss   STATUS,Z    ;if higher bits zero, not IIR mode
        goto    iir         ;branch out to use IIR filter
        bsf     FrstPss,0   ;reset IIR init flag in case we ever renter IIR
        call    FI2B        ;restore B (locktest takes |phase diff|)
        call    B2C         ;error signal direct to dac mode
        call    mulCStr     ;multiply the error value
	
    ifdef WIDE_PHASE
        btfsc   Wide,0      ;test flag - branch if set
        goto    mulC3       ;x8 - set so max output +/- 3v for Ts=2 sec
    endif                   ;WIDE_PHASE

        goto    mulC5       ;x32 - set so max output +/- 3v for Ts=30 sec
c2dac

    ifdef ALPHA_FILTER
        bcf     UsedAlf,0   ;clear flag
        movf    cLoBits,W   ;use alpha filter only if N=4,5,6,7
        andlw   0x04
        btfss   STATUS,Z    ;4 bit must be set, else skip alpha filter 
        call    alpha       ;compute alpha, return result (or original IIR) in C  
    endif                   ;ALPHA_FILTER

        movf    Hdac,W      ;get previous dac value
        movwf   Hbyte       ;store in BCD regs
        movf    Ldac,W
        movwf   Lbyte

    ifndef ALPHA_FILTER	    ;DIP-selected open loop option unavailable with alpha filter
        btfsc   dipbits,4   ;user input = 1 means Open loop (don't change DAC)
        goto    sendpop	    ;skip the call to setDAC
    endif                   ;ALPHA_FILTER

    ifdef ASCII_INPUT
        btfsc   S_hold      ;ASCII command can always select open loop
        goto    sendpop     ;skip the call to setDAC
        btfsc   JamEFC      ;direct set of EFC?
        call    jam_ascii   ;yes - set DAC with EFC 
    endif                   ;ASCII_INPUT

        movf    Hc,W        ;save input for later BCD conversion & in Hdac,Ldac
        movwf   Hbyte
        movwf   Hdac
        movf    Mc,W
        movwf   Lbyte
        movwf   Ldac
        call    compC18     ;Dac needs 2s compliment data
        call    setDAC      ;send value to the DAC
sendpop call    print       ;print filter output (DAC value to set VCO input voltage)
stordat call    Data2pd     ;save current phase data as previous data
        call    prntUSt     ;print mode input pins and status LEDS
        movf    devID,W     ;print ID character if not space or NULL
        btfsc   STATUS,Z 
        goto    $ + 6
        sublw   0x20        ;compare to space
        btfsc   STATUS,Z
        goto    $ + 3
        movf    devID,W     ;print ID if not space or NULL
        call    xmt

;reset receiver every 30 seconds just prior to CRNL to assure no port hang (bug)
;control programs can send just after CRNL assured of no conflict.
;inline serial clear avoids critical section around rcverr.

        bcf     RCSTA,CREN  ;clear overrun error by
        bsf     RCSTA,CREN  ;disable/enable rcv
        movf    RCREG,0     ;get char from input buffer
        call    prtCRNL     ;print \r \n to end a line of output values 

;end of the tasty business (wherein most work was done by the following routines) 

pop     bcf     STATUS,RP0  ;"pop" instructions (see AN611)
        swapf   STAT_TEMP,W
        movwf   STATUS
        swapf   W_TEMP,F
        swapf   W_TEMP,W
        retfie              ;return from interrupt

;*********************************************************************
;                            Subroutines
;*********************************************************************
;alpha filter - an exponential smoothing LP filter with HF cutoff above the
;cutoff of the IIR filter. If used it will further reduce noise in the 
;frequency region where the IIR has already reached its max attenuation.
;The output O(n) is computed from the previous output O(n-1) and the current 
;input I(n) by O(n) = alpha*I(n) + (1-alpha)*O(n-1). A gentler alpha (1/16 
;instead of 1/32) can be had by replacing RshftB5 by RshftB4 in 2 places below.     

    ifdef ALPHA_FILTER
alpha
    ifdef ALPHA_FILTER_DEBUG_PRINT
        movf    Hc,W        ;debug - print incoming value from IIR filter
        movwf   Hbyte
        movf    Mc,W
        movwf   Lbyte
        call    print       ;end of debug print
    endif                   ;ALPHA_FILTER_DEBUG_PRINT

        btfsc   dipbits,4   ;user input: 1 (open) means use the post filter
        goto    alpha1      ;branch to use filter
        bsf     FrstAlf,0   ;otherwise set init flag in case we use later
        return

alpha1  bcf     PORTA,3     ;turn LED off - alpha filter overwrites locktst LED
        btfsc   FrstAlf,0   ;first use?
        goto    initAlf     ;yes - init AOL storage and merely return input
        call    AOL2B       ;get previous filter output
        call    RshftB5     ;alpha = 1/32, B has alpha * O(n-1)
;       call    RshftB4     ;alpha = 1/16
        call    B2A         ;save in A
        call    C2B         ;get incoming value I(n)
        call    RshftB5     ;B has alpha * I(n)
;       call    RshftB4     ;alpha = 1/16
        call    Sub24       ;B - A -> B, B has alpha(I(n)-O(n-1))
        call    B2A         ;save in A
        call    AOL2B       ;get O(n-1) again
        call    Add24       ;B has O(n-1) + alpha(I(n)-O(n-1))
        call    cmpAlf      ;is alpha result OK? c.f. with IIR result (in C)
;   	call    B2C         ;return with final result in C (like IIR)
initAlf call    C2AOL       ;also save as O(n-1)
        clrf    FrstAlf     ;clear the first pass flag in case it's set
        return

cmpAlf  call    B2AOC       ;save current computed alpha result
;       call    printB      ;debug
        call    C2A         ;IIR result to A
        call    Sub24       ;alpha value-IIR value in B
        btfsc   Hb,7        ;want |alpha-IIR| so change sign if negative
        call    compB
;       call    printB      ;debug
        clrf    Ha          ;is difference < 50? (about 4.3 mv, or 1 nsec/30 sec)
        clrf    La
        movlw   D'50'       ;note the constant 34 is on the 16-bit scale so
        movwf   Ma          ;we stick it 2 bits up from the bottom of the 18 DAC word  
        call    Sub24
;       call    printB      ;debug
        btfsc   Hb,7        ;yes - (diff-34 is negative) use alpha result
        goto    AlfOK
        bcf     UsedAlf,0   ;no - clear flags
        bsf     PORTA,3     ;turn LED on to indicate alpha rejected
        bsf     FrstAlf,0   ;force reinitialize of alpha filter (should we?)
        return              ;return with IIR result still in C
AlfOK   bsf     UsedAlf,0   ;set flag
        call    AOC2C       ;replace IIR result
        return              ;return with alpha result in C

    endif                   ;ALPHA_FILTER
    
;*********************************************************************
;Note  This routine was extensively modified by Richard McCorkle
;as the original progressed too quickly at the longer filter times.
;When enabled the mode is set by the switches at startup and if the
;IIR is selected (mode is > 1) the Auto_N feature advances the mode 
;up to the Nmax value over time. Entering an Nmax value > 1 starts
;the Auto_N routine and entering an Nmax value of 0 disables it. 
;  When the phase error exceeds +/- 20 counts the advance timer is 
;reset to 0, when less than +/- 20 counts the advance timer is 
;incremented. When the timer exceeds 8 consecutive samples in mode 2 
;(Tau = 240) with an error of < +/- 20 counts the filter is advanced 
;and the timer is reset. In each subsequent mode the number of 
;samples doubles before the mode is advanced to increase the settle 
;time before advance to match the current filter Tau. The advance 
;from mode 6 takes 128 consecutive samples (Tau = 3840) with an 
;error of < +/- 20 counts before advancing to mode 7. 

Auto_N  movf    Nmax,W	    ;if Nmax < 2 (disabled) 		
        andlw   0x06
        btfsc	STATUS,Z    ;just exit
        return              ;(not used in setup and direct modes)
        movf    dipbits,W   ;get mode		
        andlw   0x07        ;mask hi bits
        subwf   Nmax,W      ;Nmax - mode in W
        btfsc   STATUS,Z    ;if mode is already at max
        return              ;just exit
        btfss   STATUS,C    ;if mode is > max
        return              ;just exit
        movlw   D'20'       ;put 20 in A (advance phase limit)
        movwf   Ma          ;B has absolute error from setpoint
        clrf    Ha          ;from the lock test routine on call
        clrf    La
        call    Sub24
        btfsc   Hb,7        ;if error > 20 
        goto    $ + 3
        clrf    Nctr        ;clear the N advance counter
        return              ;and exit
        incf    Nctr        ;else increment the advance counter
        movlw   D'8'        ;advance count threshold to temp
        movwf   temp
        movf    dipbits,W   ;get mode		
        andlw   0x07        ;mask hi bits
        addlw   0xfe        ;subtract 2 from mode for multiplier
        btfsc   STATUS,Z    ;if multiplier is 0 
        goto    $ + 6       ;skip multiply
        bcf     STATUS,C    ;multiply threshold * (mode - 2)
        rlf     temp
        addlw   0xff
        btfss   STATUS,Z
        goto    $ - 4			
        subwf   Nctr,W      ;subtract from counter
        btfss   STATUS,Z    ;if counter < threshold 
        return              ;exit
        movf    dipbits,W   ;get mode		
        movwf   diptemp     ;and save in diptemp	
        incf    diptemp     ;advance the mode 1 step 
        bsf     diptemp,7   ;set bit 7 like serial entry		
        clrf    Nctr        ;reset advance counter
        return              ;and exit

;*********************************************************************
;clear the counter and previous phi storage

clr166  bsf     PORTB,4     ;toggle i/o pin to clear counter
        bcf     PORTB,4
        clrf    Hphip       ;clear phiwatch previous phi storage
        clrf    Lphip
        return

;*********************************************************************
;copy physical DIP switch settings to the shadow - can be overwritten by ASCII 

copydip	movf    dipbits,W
        andlw   0xc8        ;clear bits shadowing physical switches
        movwf   dipbits
        movf    PORTC,W     ;add 3 low bits of port C (mode)
        andlw   0x07
        addwf   dipbits
        btfsc   PORTB,2     ;set Alpha bit 
        bsf     dipbits,4
        btfsc   PORTB,1     ;set polarity bit 
        bsf     dipbits,5
        return

;*********************************************************************
;ignore current phase data, d[i], if d[i] - d[i-1] is too large.  
;Use d[i-1] instead. To avoid hangups in case of valid phase offset 
;use only 3 times in a row. set glitch LED.

dGlitch
    ifdef ASCII_INPUT
        btfsc   dGlOff      ;is there an ASCII comd to disable deGlitcher?	
        goto    clrret      ;yes - just return
    endif                   ;ASCII_INPUT		
        movf    cLoBits,W   ;use only in slower iir filter modes: 4,5,6,7
        andlw   0x04        ;fast modes make large steps that conflict      
        btfsc   STATUS,Z    ;test mode bits, 4 bit must be set
        goto    clrret      ;just return, we're not in N>3 mode
        call    Data2B      ;begin computation of d[i]-d[i-1]
        call    pd2A
        call    Sub24       ;current-previous in B
        btfsc   Hb,7        ;want |diff|
        call    compB       ;so if neg make it positive
        movlw   D'30'       ;glitch limit constant
        movwf   Ma          ;into A low bits of 16 bit word
        clrf    Ha
        clrf    La
        call    Sub24       ;|diff|-limit in B
        btfsc   Hb,7        ;negative?
        goto    clrret      ;yes, within limit, nothing to do
        incf    glcctr      ;increment successive glitch counter
        btfsc   glcctr,2    ;test counter for >3 in-a-row
        goto    clrret      ;too many in-a-row, don't overwrite current value
        movf    Hpdat,W     ;otherwise get d[i-1]
        movwf   Hdata       ;and overwrite d[i]
        movf    Lpdat,W     ;note: this will cause d[i-1] to remain
        movwf   Ldata       ;unchanged on successive entries
        bsf     PORTA,1     ;turn glitch LED on
        return              ;hence the need for the counter
clrret  clrf    glcctr      ;clear counter and return
        bcf     PORTA,1     ;turn glitch LED off (if on)
        return
 
;*********************************************************************
;IIR filter (digital vsn of amplified lead/lag) - expects current 
;input in B parameters are Ts=30, F2=64, F1 and Kcpu selected by user 
;pins (PORTC 0,1,2) fo=fol + (1/F1 + 1/F2)*fi + (1/F1 - 1/F2)*fil
;The input data is placed in the top two bytes of the 40-bit math reg 
;to allow division w/o losing accuracy. Output is taken from the top 
;18 bits. The filter gain is controlled by left-shifting the output 
;value before sending to the DAC. 

iir     decfsz  toc         ;filter sample time is "minutes" minutes
        goto    nottime     ;not time yet
        movf    minutes,W   ;reset toc counter
        movwf   toc
        btfsc   FrstPss,0   ;first time thru the filter?
        call    initIIR     ;yes  - init previous value storage
        call    scale       ;check if Tc was changed since last entry
        call    FI2B        ;get current filter input: i(n)
        call    RshBL6      ;i(n)/64
        call    B2A         ;tranfer to A for add
        call    divByF1     ;i(n)/F1 - note: already divided by 64
        call    Add40       ;B has i(n)*(1/F1 + 1/64)
;       call    prntB32     ;debug
        call    FOL2A       ;get o(n-1) - last filter output
        call    Add40       ;B has o(n-1)+i(n)*(1/F1 + 1/64)
;       call    prntB32     ;debug
        call    B2C         ;save in C for a bit
        call    FIL2B       ;get i(n-1) - last filter input
;       call    prntB32     ;debug
        call    RshBL6      ;i(n-1)/64
        call    B2A         ;transfer to A for subtraction
        call    divByF1     ;i(n-1)/F1 - note: already divided by 64
        call    Sub40       ;B has i(n-1)*(1/F1 - 1/64), a neg value
;       call    prntB32     ;debug
        call    C2A         ;get previous partial sum
        call    Add40       ;o(n-1)+i(n)*(1/F1+1/64)+i(n-1)*(1/F1-1/64)
;       call    prntB32     ;debug
        call    B2C         ;save for output to dac, etc
        call    B2FOL       ;also save as o(n-1)
;       call    prntB32     ;debug
        call    FI2FIL      ;move i(n) to i(n-1)
        call    mulCStr     ;initialize multiply routine that sets loop gain
        goto    mul_IIR     ;multiply C and return
        
initIIR call    B2ILOL      ;copy current error sig to previous i, clear o
        movf    cLoBits,W   ;initialize previous filter mode storage
        movwf   pLoBits
        clrf    FrstPss
        return

;*********************************************************************
;Note  The limiter was revised by Richard McCorkle to correct a 
;WIDE_PHASE conditional error and to use the actual setpoint in use. 
;A new BP_LIMIT conditional and bipolar limiter were added for use 
;with setpoints > 1023.   

;  In the original code if WIDE_PHASE was not defined then the limiter 
;label was present but no limiter was included in the compiled hex 
;file and locktst and Auto_N were called twice. The code has been 
;revised so a limiter is included when WIDE_PHASE is not defined. 
;  In earlier versions the setpoint was fixed at 800 so the limit was 
;fixed at 1823 so any value greater than the setpoint + 1023 was 
;limited to prevent a sign reversal after the 32x multiply in N=1. 
;With the default setpoint change to 780 the limit should have been 
;reduced to 1803 for proper limiting. With the addition of the ASCII 
;setpoint routine the user can now vary the setpoint, so using a fixed 
;limit won't guarantee the error will be limited properly to prevent 
;a sign reversal at large phase offsets. The original WIDE_PHASE 
;limiter is used but the standard limiter uses a limit based on the 
;actual setpoint in use.  
;  A setpoint greater than 1023 can also be entered which would allow 
;a < -1023 error with a negative overflow and sign reversal. The 
;maximum setpoint entered using the original code should be 1023 or 
;less to prevent a negative overflow. A new BP_LIMIT conditional and 
;bipolar limiter were added for use with setpoints > 1023.


;limits the max input from the phase detector so low freq VCOs do not
;cause a large data input values to the filter. Called before setpoint
;is subtracted. Note that N=1 mode multiplies by 32 and 32x1024 appears 
;neg. so limiter must prevent phase-setpoint values >1023. Not a 
;problem for wide phase (merely x8, not x32).

limiter
    ifdef WIDE_PHASE
        btfss   Wide,0	    ;test flag and shift only wide phase data
        goto    lim8
        call    Data2A      ;if data >= 1600+1500, set to 3100 (0xC1C)
        movlw   0xC         ;load 0xC1C into B (24 bits)
        movwf   Hb
        movlw   0x1C
        movwf   Mb
        clrf    Lb
        call    Sub24       ;B-A->B  (computes 3100 - phase)
        btfss   Hb,7        ;is the difference negative?
        return              ;no - data is OK
        movlw   0xC         ;otherwise make it 3100
        movwf   Hdata
        movlw   0x1C
        movwf   Ldata
        return
    endif                   ;WIDE_PHASE

    ifdef BP_LIMIT          ;Bipolar limiter for setpoints > 1023

;If data > setpoint do pos test else do neg test

Lim8    call    Data2B      ;data to B
        movf    Lsp,W       ;subtract setpoint
        subwf   Mb
        btfss   STATUS,C
        decf    Hb
        movf    Hsp,W
        subwf   Hb
        btfsc   Hb,7        ;if data < setpoint check neg
        goto 	NegL
PosL    movlw   0xff        ;subtract 1023 limit
        subwf   Mb
        btfss   STATUS,C
        decf    Hb
        movlw   0x03
        subwf   Hb
        btfsc   Hb,7        ;if neg data < limit
        return              ;data OK so return
        movf    Lsp,W       ;else setpoint to data
        movwf   Ldata
        movf    Hsp,W
        movwf   Hdata
        movlw   0xff        ;add 1023 limit
        addwf   Ldata
        btfsc   STATUS,C
        incf    Hdata
        movlw   0x03
        addwf   Hdata
        return              ;exit with SP + 1023 in data
NegL    movlw   0xff        ;add 1023 limit to (data - setpoint)
        addwf   Mb
        btfsc   STATUS,C
        incf    Hb
        movlw   0x03
        addwf   Hb
        btfss   Hb,7        ;if pos data is > (setpoint - limit)
        return              ;data OK so return
        comf    Mb          ;2's comp for pos offset needed
        comf    Hb
        incf    Mb
        btfsc   STATUS,Z
        incf    Hb
        movf    Mb,W        ;add offset to data 
        addwf   Ldata 
        btfsc   STATUS,C
        incf    Hdata
        movf    Hb,W
        addwf   Hdata
        return              ;exit with SP - 1023 in data		

    else                    ;original limiter for setpoints < 1023

lim8    call    Data2A      ;if data >= SP+1024, set to SP+1023
        movlw   0x3         ;load 0x3ff (1023 limit) in B (24 bits)
        movwf   Hb
        movlw   0xff
        movwf   Mb
        clrf    Lb
        movf    Lsp,W       ;add setpoint for adjusted limit	
        addwf   Mb
        btfsc   STATUS,C
        incf    Hb
        movf    Hsp,W
        addwf   Hb
        call    Sub24       ;B-A->B  (computes limit - phase)
        btfss   Hb,7        ;is the difference negative?
        return              ;no - data is OK
        call    Sub24	    ;restore limit in B 
        movf    Hb,W        ;(A is neg from 1st sub)
        movwf   Hdata       ;store limit in Data
        movf    Mb,W
        movwf   Ldata
        return

    endif                   ;BP_LIMIT

;***************************************************************
;test for phase reading near setpoint: an indicator (over time) 
;that PLL is locked

locktst call    Data2B      ;get current phase reading
        call    SP2A        ;get setpoint
        call    Sub24       ;phase-setpoint in B
        btfsc   Hb,7        ;we want |data-sp|, so change sign if negative
        call    compB
        bcf     PORTA,3     ;turn LED off first 
        movf    Hb,W        ;get high byte of |diff|
        btfss   STATUS,Z    ;any bits set means |diff| > 255
        bsf     PORTA,3     ;so turn on LED
        goto    Auto_N      ;to automatically increase N value 
;       return              ;per phase diff if enabled
 
;*********************************************************************
;call me to burn 4 instruction cycles

nop4    return              

;*********************************************************************
;print last DAC if not update time

nottime movf    Hdac,W      ;not IIR sample time yet
        movwf   Hbyte       ;just copy old dac value for ascii ouput
        movf    Ldac,W
        movwf   Lbyte
        goto    sendpop     ;jump back to output ascii and retfie

;***************************************************************
;phi_watch reads the phase each second and reports via a PWM-driven analog 
;output.  Thus the user can observe sec-to-sec phase changes and quickly
;set a VCXO close to frequency. Concept & original code by Bob Larkin, W7UPA 

phiwtch 
    ifdef INT_CTR           ;read data from internal counter and prescaler
        call    readctr     
    else                    ;or 
        call    read166     ;read 166 for current phase to Hbyte, Lbyte
    endif                   ;INT_CTR
        call    phi2A       ;get previous reading
        call    Dat2phi     ;save new reading as previous
        call    Data2B      ;get new reading to B
        call    Sub24       ;(new reading - previous) in B

    ifdef WIDE_PHASE
        btfsc   Wide,0      ;test flag and shift only wide phase data
        call    RshftB5     ;divide by 32 to get data in lo byte
    endif                   ;WIDE_PHASE

        movf    Mb,W        ;save 8 low bits for PWM code
        movwf   phi

;want phi/phic(ON/OFF)PWM ratio so setpoint counts/sec, S=(1600/32)/2
;produces 1V out of 5V. Since C+S=5 this determines C such that 
;ON/OFF=S/(C-S)=1/4,i.e., C=5S=5x25=125=0x7D. The incoming data,
;a value near, 25,is subtracted from C to produce the OFF time  
;note: for 800 setpoint, C is about 127(0x7f), use 127 for both 
;setpoints

        sublw   0x7F        ;0x7f (127.)
        movwf   phic        ;phic has (0x7f-phi)
        return

;*********************************************************************
;read external counter (shift register) to get phase data

read166
    ifdef WIDE_PHASE
        btfss   Wide,0	    ;test flag
        goto    rd166       ;no need to stall if not wide phase
        movlw   D'60'       ;stall for about 120 usec so phase counter 
        movwf   temp        ;finishes before loading shift registers
stall   decfsz 	temp
        goto    stall
    endif                   ;WIDE_PHASE

rd166   bcf     PORTB,3     ;set '166 shift/load pin to load
        call    read_spi    ;just to toggle the '166 clock (8 times!)
        bsf     PORTB,3     ;set '166 pin to shift to get data
        call    read_spi
        movwf   Hdata       ;save the hi 8 bits
        movwf   Hbyte       ;save again for B_2_BCD
        call    read_spi
        movwf   Ldata       ;ditto for lo 8 bits
        movwf   Lbyte
        return
    
;*********************************************************************
;  When the 166 is read the counter is not reset and continues
;accumulating until the clr166 reset. When the readctr routine
;was developed the phiwatch code was not yet implemented so it
;was only called at update and contained the accumulated sample. 
;With the addition of phiwatch the readctr routine is called 
;each second and the counter is reset every sample after the 
;read. There is no accumulation of the value so the routine 
;needs to be modified to work with phiwatch.
;  I have added a 16-bit accumulator register that gets cleared
;after each update. The samples are added to the accumulator at 
;each read, then the accumulator value is transferred to emulate 
;the operation of the 166 routine.    

;read internal prescaler and counter to get phase data (1.5 msec max 
;@ 6 MHz) RA4 (counter input) tied to RA5, and to input source 
;(NAND gate with XO and phase gate inputs) via 470 ohm resistor

readctr movf    TMR0,W      ;get main counter value (8 MSBs)
        movwf   Hdata       ;save
        addwf   Hic         ;add hi bits to accumulator
        bsf     STATUS,RP0  ;set bank 1 to access tris
        bcf     TRISA,5     ;make RA5 an output
        bcf     STATUS,RP0  ;back to bank 0
        clrf    temp        ;clear loop counter
toggle  incf    temp,1      ;increment prescaler until overflow (AN592) 
        bcf     PORTA,5     ;toggle pin connected to count (prescaler) input
        bsf     PORTA,5
        movf    TMR0,W      ;check for overflow to main counter
        subwf   Hdata,W     ;subtract earlier reading
        btfsc   STATUS,Z    ;if tmr0 incremented get out of loop
        goto    toggle      ;otherwise toggle again
        comf    temp,1      ;complement number of toggles needed
        incf    temp,W      ;+1
        addwf   Lic         ;add low bits to accumulator
        btfsc   STATUS,C    ;if carry
        incf    Hic         ;inc hi bits
        bsf     STATUS,RP0  ;set bank 1 to access tris
        bsf     TRISA,5     ;make RA5 an input again to release counter
        bcf     STATUS,RP0  ;back to bank 0
        clrf    TMR0        ;clear the counter & prescaler
        movf    Lic,W       ;accumulator low bits to W
        movwf   Ldata       ;save low bits  
        movwf   Lbyte       ;save again for B_2_BCD routine
        movf    Hic,W       ;accumulator hi bits to W
        movwf   Hdata       ;save hi bits  
        movwf   Hbyte       ;save again for B_2_BCD routine
        return

;***************************************************************
;subroutine to read data from the SPI port, return in W

read_spi
        movwf   SSPBUF      ;initiate a read cycle by writing SSPBUF
        bsf     STATUS,RP0  ;set bank1 to access SSPSTAT
loopspi btfss   SSPSTAT,BF  ;test i/o complete
        goto    loopspi     ;wait - not ready yet
        bcf     STATUS,RP0  ;back to bank 0
        movf    SSPBUF,W    ;get data byte into W
        return

;***************************************************************
;output the 18-bit word in C reg to a Burr-Brown PCM-61P DAC

setDAC  movlw   D'17'       ;setup loop counter
        movwf   temp
        ;reset latch only after next MSB - funny business for the PCM-61
        call    ssc         ;shift out next bit, set data line, clock
        bsf     PORTB,7     ;set latch hi AFTER the first bit of the new word
dloop                       ;now do the lower 17 bits   
        call    ssc         ;shift out next bit, set data line, clock
        decfsz  temp,1      ;dec loop count
        goto    dloop       ;loop 18 times
        bcf     PORTB,7     ;to set new value into DAC set latch lo...
        return              ;note - this routine destroys data in C
ssc     call    LshiftC     ;shift input reg: hi bits out first
        btfss   STATUS,C    ;to equalize timing - remove later
        nop         	    ;(only makes a better scope display)
        btfsc   STATUS,C    ;was top bit a one?
        bsf     PORTB,5     ;yes - set data line hi
        bsf     PORTB,6     ;data now set toggle clock line
        bcf     PORTB,6
        bcf     PORTB,5     ;set data out line lo again
        return

;***************************************************************
;setup mode - assist user in coarse adjustment of VCXO frequency
        
setupMd clrf    temp2       ;setup mode - turn on LEDS when phase is
        clrf    Hc          ;increasing (freq lo) or decreasing (freq hi)
        clrf    Mc          ;set DAC to zero to center VCO
        clrf    Lc          ;(also useful to measure DAC offset)
        call    setDAC
        bcf     PORTA,1     ;initially set both off
        bcf     PORTA,3
        call    Data2B      ;get current phase data
        call    pd2A        ;get previous phase
        call    Sub24       ;(current - previous) phase in B
        btfss   Hb,7        ;is difference negative?
        goto    dpl         ;no - branch around negative case stuff
        incf    temp2       ;yes - set sign flag for later use
        call    compB       ;and make it positive, i.e. we have |B| now
dpl    	movlw   D'60'       ;limit value, |diff| > limit means turn an LED on
        movwf   Ma          ;limit to A reg
        clrf    Ha
        clrf    La
        call    Sub24       ;B - A -> B
        btfsc   Hb,7        ;if negative |diff| is within limits
        goto    stordat     ;so just go back to interrupt loop
        btfsc   temp2,0     ;outside limits, check direction
        goto    hiLED       ;freq too hi, jump
        bsf     PORTA,1     ;freq too lo - set lo LED on
        goto    stordat     ;back to interrupt loop
hiLED   bsf     PORTA,3     ;freq too hi - set hi LED on
        goto    stordat

;*********************************************************************
;user-selected initialize routine (selected by 3 lo bits (value 0-7) in PORTC

usr0                        ;special case if needed for setup mode
usr1
usr2
usr3
usr4
usr5
usr6        
usr7
        movlw   D'30'       ;sample time 30 sec
    ifdef WIDE_PHASE
        btfsc   Wide,0	    ;test flag and over-write if appropriate
        movlw   D'2'        ;sample time 2 sec
    endif                   ;WIDE_PHASE
        movwf   tsecs       ;store selected value into tsecs
        return          

;***************************************************************
;                         Move routines
;***************************************************************
;Data transfer routines used in IIR, etc

A2C     movf    Ha,W        ;transfer 40 bits A reg to C reg
        movwf   Hc
        movf    Ma,W
        movwf   Mc
        movf    La,W
        movwf   Lc
        movf    Sa,W
        movwf   Sc
        movf    SSa,W
        movwf   SSc
        return

AOC2C   movf    Haoc,W      ;transfer 24 bits from alpha filter current to C
        movwf   Hc
        movf    Maoc,W
        movwf   Mc
        movf    Laoc,W
        movwf   Lc
        return      

AOL2B   movf    Haol,W      ;transfer 24 bits from alpha filter last to B
        movwf   Hb
        movf    Maol,W
        movwf   Mb
        movf    Laol,W
        movwf   Lb
        return

B2A     movf    Hb,W        ;transfer 40 bits B reg to A reg
        movwf   Ha
        movf    Mb,W
        movwf   Ma
        movf    Lb,W
        movwf   La
        movf    Sb,W
        movwf   Sa
        movf    SSb,W
        movwf   SSa
        return

B2AOC   movf    Hb,W        ;transfer 24 bits from B to alpha filter current
        movwf   Haoc
        movf    Mb,W
        movwf   Maoc
        movf    Lb,W
        movwf   Laoc
        return

B2C     movf    Hb,W        ;transfer 40 bits B reg to C reg
        movwf   Hc
        movf    Mb,W
        movwf   Mc
        movf    Lb,W
        movwf   Lc
        movf    Sb,W
        movwf   Sc
        movf    SSb,W
        movwf   SSc
        return

B2FIBCD movf    Hb,W        ;transfer B reg to filter input and
        movwf   Hfi         ;to Bin2BCD work regs (2 bytes)
        movwf   Hbyte
        movf    Mb,W
        movwf   Lfi
        movwf   Lbyte
        return

B2FOL   movf    Hb,W        ;transfer 40 bits B reg to previous output
        movwf   Hfol
        movf    Mb,W
        movwf   Mfol
        movf    Lb,W
        movwf   Lfol
        movf    Sb,W
        movwf   Sfol
        movf    SSb,W
        movwf   SSfol
        return

B2ILOL  movf    Hb,W        ;xfer B reg to previous input, clr prev. output
        movwf   Hfil
        movf    Mb,W
        movwf   Lfil
        clrf    Hfol
        clrf    Mfol
        clrf    Lfol
        clrf    Sfol
        clrf    SSfol
        return

C2A     movf    Hc,W        ;transfer 40 bits C reg to A reg
        movwf   Ha
        movf    Mc,W
        movwf   Ma
        movf    Lc,W
        movwf   La
        movf    Sc,W
        movwf   Sa
        movf    SSc,W
        movwf   SSa
        return

C2AOL   movf    Hc,W        ;transfer 24 bits from C to alpha filter last
        movwf   Haol
        movf    Mc,W
        movwf   Maol
        movf    Lc,W
        movwf   Laol
        return

C2B     movf    Hc,W        ;transfer 40 bits C reg to B reg
        movwf   Hb
        movf    Mc,W
        movwf   Mb
        movf    Lc,W
        movwf   Lb
        movf    Sc,W
        movwf   Sb
        movf    SSc,W
        movwf   SSb
        return

Data2A  movf    Hdata,W     ;transfer Data (2 bytes) to A reg
        movwf   Ha
        movf    Ldata,W
s_c_r   movwf   Ma
        clrf    La
        clrf    Sa
        clrf    SSa
        return
   
Data2B  movf    Hdata,W     ;transfer Data (2 bytes) to B reg
        movwf   Hb
        movf    Ldata,W
        movwf   Mb
        clrf    Lb
        clrf    Sb
        clrf    SSb
        return

Data2pd movf    Hdata,W     ;transfer Data (2 bytes) to previous data store
        movwf   Hpdat
        movf    Ldata,W
        movwf   Lpdat
        return

Dat2phi movf    Hdata,W     ;transfer Data (2 bytes) to phi storage
        movwf   Hphip
        movf    Ldata,W
        movwf   Lphip
        return

EFC2A   movf    Hefc,W      ;transfer ASCII input EFC value to A reg (16 bits only)
        movwf   Ha
        movf    Lefc,W
        movwf   Ma
        goto    s_c_r       ;store, clear and return

FI2A    movf    Hfi,W       ;filter input to A (2 bytes)
        movwf   Ha
        movf    Lfi,W
        movwf   Ma
        clrf    La
        clrf    Sa
        clrf    SSa
        return

FI2B    movf    Hfi,W       ;filter input to B (2 bytes)
        movwf   Hb
        movf    Lfi,W
        movwf   Mb
        clrf    Lb
        clrf    Sb
        clrf    SSb
        return

FIL2B   movf    Hfil,W      ;previous filter input to B (2 bytes)
        movwf   Hb
        movf    Lfil,W
        movwf   Mb
        clrf    Lb
        clrf    Sb
        clrf    SSb
        return

FI2FIL  movf    Hfi,W       ;filter input to previous filter input (2 bytes)
        movwf   Hfil
        movf    Lfi,W
        movwf   Lfil
        return

FOL2A   movf    Hfol,W      ;transfer 40 bits previous output to A reg
        movwf   Ha
        movf    Mfol,W
        movwf   Ma
        movf    Lfol,W
        movwf   La
        movf    Sfol,W
        movwf   Sa
        movf    SSfol,W
        movwf   SSa
        return

pd2A    movf    Hpdat,W     ;transfer previous data (2 bytes) to A reg
        movwf   Ha
        movf    Lpdat,W
        movwf   Ma
        clrf    La
        clrf    Sa
        clrf    SSa
        return

phi2A   movf    Hphip,W     ;transfer previous phi (2 bytes) to A reg
        movwf   Ha
        movf    Lphip,W
        goto    s_c_r       ;store, clear, and return
    
SP2A    movf    Hsp,W       ;transfer phase setpoint value (16 bits) to A reg
        movwf   Ha
        movf    Lsp,W
        movwf   Ma
        clrf    La
        clrf    Sa
        clrf    SSa
        return

SP2B    movf    Hsp,W       ;transfer phase setpoint value (16 bits) to B reg
        movwf   Hb
        movf    Lsp,W
        movwf   Mb
        clrf    Lb
        clrf    Sb
        clrf    SSb
        return

;***************************************************************
;                         Math routines
;***************************************************************
;24-bit and 40-bit math routines (see AN611) 

Add24   movf    La,W        ;A + B -> B  (24 bits)
        addwf   Lb          ;add low bytes
        btfsc   STATUS,C    ;add in carry if necessary
        goto    add2
add1    movf    Ma,W
        addwf   Mb          ;add mid bytes
        btfsc   STATUS,C    ;add in carry if necessary
        incf    Hb
        movf    Ha,W
        addwf   Hb
        return
add2    incf    Mb
        btfsc   STATUS,Z
        incf    Hb
        goto    add1

Add40   movf    SSa,W       ;A + B -> B  (40 bits)
        addwf   SSb         ;add low bytes
        btfsc   STATUS,C    ;add in carry if necessary
        call    addl3
        movf    Sa,W
        addwf   Sb
        btfsc   STATUS,C
        call    addl2
        movf    La,W
        addwf   Lb
        btfsc   STATUS,C
        call    addl1
        movf    Ma,W
        addwf   Mb
        btfsc   STATUS,C
        incf    Hb
        movf    Ha,W
        addwf   Hb
        return

addl3   incf    Sb          ;when Z is clear skip the rest of the chain
        btfsc   STATUS,Z
addl2   incf    Lb
        btfsc   STATUS,Z
addl1   incf    Mb
        btfsc   STATUS,Z
        incf    Hb
        return

compA   comf    La          ;2s complement of A -> A
        comf    Ma          ;invert all the bits in A
        comf    Ha
        incf    La          ;add one to A
        btfsc   STATUS,Z
        incf    Ma
        btfsc   STATUS,Z
        incf    Ha
        return

compA40 comf    SSa         ;2s complement of A -> A
        comf    Sa          ;invert all the bits in A
        comf    La
        comf    Ma
        comf    Ha
        incf    SSa         ;add one to A
        btfsc   STATUS,Z
        incf    Sa
        btfsc   STATUS,Z
        incf    La
        btfsc   STATUS,Z
        incf    Ma
        btfsc   STATUS,Z
        incf    Ha
        return

compB   comf    Lb          ;2s complement of B -> B
        comf    Mb          ;invert all the bits in B
        comf    Hb
        incf    Lb          ;add one to B
        btfsc   STATUS,Z
        incf    Mb
        btfsc   STATUS,Z
        incf    Hb
        return

compB40 comf    SSb         ;2s complement of B -> B
        comf    Sb          ;invert all the bits in B
        comf    Lb
        comf    Mb
        comf    Hb
        incf    SSb         ;add one to B
        btfsc   STATUS,Z    ;a skip chain
        incf    Sb
        btfsc   STATUS,Z
        incf    Lb
        btfsc   STATUS,Z
        incf    Mb
        btfsc   STATUS,Z
        incf    Hb
        return

compC   comf    Lc          ;2s complement of C -> C
        comf    Mc          ;invert all the bits in C
        comf    Hc
        incf    Lc          ;add one to C
        btfsc   STATUS,Z
        incf    Mc
        btfsc   STATUS,Z
        incf    Hc
        return

compC18 comf    Lc          ;2s compliment of 18 bit value in C -> C
        comf    Mc          ;invert all the bits in A
        comf    Hc
        movlw   0x40        ;add 1 (18th bit, not 24th!)
        addwf   Lc,1
        btfsc   STATUS,Z
        incf    Mc
        btfsc   STATUS,Z
        incf    Hc
        return

compC40 comf    SSc         ;2s complement of C -> C
        comf    Sc          ;invert all the bits in C
        comf    Lc
        comf    Mc
        comf    Hc
        incf    SSc         ;add one to C
        btfsc   STATUS,Z    ;a skip chain
        incf    Sc
        btfsc   STATUS,Z
        incf    Lc
        btfsc   STATUS,Z
        incf    Mc
        btfsc   STATUS,Z
        incf    Hc
        return

LshiftC bcf     STATUS,C    ;left-shift 24 bit C reg by 1
        rlf     Lc,1
        rlf     Mc,1
        rlf     Hc,1
        return

LshfC40 bcf     STATUS,C    ;left-shift 40 bit C reg by 1
        rlf     SSc,1
        rlf     Sc,1
        rlf     Lc,1
        rlf     Mc,1
        rlf     Hc,1
        return

mulCStr btfss   Hc,7        ;set up for signed multiply of value in C reg
        return              ;value is positive, just return
        bsf     negFlag,0   ;negative - set flag
        call    compC40     ;make positive
        return

mulC11  call    LshfC40     ;multiply by shifting left
mulC10  call    LshfC40     ;enter at n, get (2^n)
mulC9   call    LshfC40     ;
mulC8   call    LshfC40
mulC7   call    LshfC40
mulC6   call    LshfC40
mulC5   call    LshfC40
mulC4   call    LshfC40
mulC3   call    LshfC40
mulC2   call    LshfC40
mulC1   call    LshfC40 
        btfsc   negFlag,0   ;was the original value negative?
        goto    negval      ;yes - branch to restore sign
        bcf     Hc,7        ;no - but make sure it's positive after shifts
        goto    mulrtn      ;return (as directed)
negval  call    compC40     ;restore sign
        bcf     negFlag,0   ;clear flag
mulrtn  btfss   sclFlag,0   ;return to IIR or scale routines as directed
        goto    c2dac       ;to IIR (usual path)
sc1     bcf     sclFlag,0   ;clear return flag
        call    C2B         ;to "scale" (only used when N is changed)
        goto    stofol

RshiftB btfsc   Hb,7        ;right-shift 24 bit B reg by 1
        goto    shfBneg     ;highest bit set - consider a neg no.       
        bcf     STATUS,C    ;clear carry flag before shifting 
        rrf     Hb          ;shift
        rrf     Mb
        rrf     Lb
        return
shfBneg call    compB       ;make a neg no positive
        bcf     STATUS,C    ;clear carry flag before shifting
        rrf     Hb          ;shift 
        rrf     Mb
        rrf     Lb
        call    compB       ;make it neg again
        return  

RshftBL btfsc   Hb,7        ;right-shift 40 bit B reg by 1
        goto    shfBLng     ;highest bit set - consider a neg no.       
        bcf     STATUS,C    ;clear carry flag before shifting 
        rrf 	Hb          ;shift
        rrf     Mb
        rrf     Lb
        rrf     Sb
        rrf     SSb
        return
shfBLng call    compB40     ;make a neg no positive
        bcf     STATUS,C    ;clear carry flag before shifting
        rrf     Hb          ;shift 
        rrf     Mb
        rrf     Lb
        rrf     Sb
        rrf     SSb
        call    compB40     ;make it neg again
        return  

RshftB5 call    RshiftB     ;right-shift B reg five times   
RshftB4 call    RshiftB     ;right-shift B reg four times
        call    RshiftB 
        call    RshiftB
        call    RshiftB
        return

RshBL12 call    RshftBL     ;right-shift long (40 bit) B reg 12 times
    	call    RshftBL
RshBL10 call    RshftBL
RshBL9  call    RshftBL 
RshBL8  call    RshftBL
RshBL7  call    RshftBL     ;right-shift long (40 bit) B reg 7 times
RshBL6  call    RshftBL     ;right-shift long (40 bit) B reg 6 times
RshBL5  call    RshftBL     ;right-shift long (40 bit) B reg 5 times
RshBL4  call    RshftBL 
        call    RshftBL
        call    RshftBL
        call    RshftBL
        return

;change iir filter data (last filter output) when user changes Tc to avoid
;a transient.  Note we only scale a step at a time, so change Tc progressively.

scale   movf    pLoBits,W   ;has user changed Tc since last entry?
        subwf   cLoBits,0   ;current - previous
        btfsc   STATUS,Z    ;are they the same?
        return              ;yes, nothing to do 
        btfsc   STATUS,C    ;test sign of difference - if negative C is zero
        goto    up          ;increasing Tc
        call    FOL2A       ;decreasing Tc - need to divide last output by two
        call    A2C
        call    C2B
        call    RshftBL     ;divide - works for both + and - values
stofol  call    B2FOL       ;store it
        movf    cLoBits,W   ;update previous setting
        movwf   pLoBits
        bsf     FrstAlf,0   ;set flag to re-initialize alpha filter
        return
up      call    FOL2A       ;increasing Tc - need to multiply last output by two
        call    A2C
        call    mulCStr     ;multiply set up - works for both + and - values
        bsf     sclFlag,0   ;set return flag (mulC routines also used by IIR)
        goto    mulC1       ;multiply by two

Sub24   call    compA       ;B - A -> B (24 bits)
        goto    Add24

Sub40   call    compA40     ;B - A -> B (40 bits)
		goto    Add40

;***************************************************************
;                        Print routines
;***************************************************************
;print code for debugging - uses only R0, R1, R2 regs

B2BCD   movf    Hb,W        ;transfer B reg 
        movwf   Hbyte       ;two hi B bytes to Bin2BCD work regs
        movf    Mb,W
        movwf   Lbyte
        return

printB  call    B2BCD       ;get B (2 bytes) to print
print   call    Bin2BCD     ;value already in work regs
        goto    sendnum		;print and return

prntB32 call    printB      ;print two 16 bit words H,M,L,S reg of B
        movf    Lb,W
        movwf   Hbyte       ;put lower bytes into print work regs
        movf    Sb,W
        movwf   Lbyte
        goto    print       ;"print" will return to caller

;print status bits

prntUSt movf    dipbits,W   ;print usr input bits and status
        andlw   0x07        ;mask user bits
        btfsc   PORTA,1     ;dglitch LED on?
        addlw   D'10'       ;if on add 10
        btfsc   UsedAlf,0   ;did we use the alpha filter?
        addlw   D'20'       ;if yes add 20.
        btfsc	dipbits,4   ;check Alpha bit (1 if open loop)
        addlw   D'50'       ;50 indicates Alpha enabled
                            ;or open loop if Alpha not defined
        btfsc   PORTA,3     ;"unlock" LED on?
        addlw   D'100'      ;if on add 100.
        movwf   Lbyte       ;print the total
        clrf    Hbyte
        goto    print

;Binary to BCD (16 Bit) Conv Routine - adapted for 16Cxx from AN544
;input in Lbyte, Hbyte and output in R2, R1, R0

Bin2BCD	bcf     STATUS,C
        clrf    count
        bsf     count,4     ;set count=16
        clrf    R0
        clrf    R1
        clrf    R2
loop16  rlf     Lbyte
        rlf     Hbyte
        rlf     R2
        rlf     R1
        rlf     R0
        decfsz  count       ;no decfsnz inst on the 16Cxx
        goto    n1
        return              ;done
n1      movlw   R2          ;load addr of R2 as indirect addr
        movwf   FSR
;adjust BCD
        movf    INDF,W      ;get R2 (later R1, R0) via indirect addr
        addlw   0x03
        movwf   wtmp        ;move sum to file reg for bit test
        btfsc   wtmp,3      ;test if result >7
        movwf   INDF        ;yes - store sum
        movf    INDF,W      ;get original or sum
        addlw   0x30        ;test hi byte
        movwf   wtmp        ;move sum to file reg for bit test
        btfsc   wtmp,7      ;test if result >0x70
        movwf   INDF        ;save as BCD
;
        incf    FSR
;
        movf    INDF,W      ;repeat for R1
        addlw   0x03
        movwf   wtmp
        btfsc   wtmp,3
        movwf   INDF
        movf    INDF,W
        addlw   0x30
        movwf   wtmp
        btfsc   wtmp,7
        movwf   INDF
;
        incf    FSR
;
        movf    INDF,W      ;repeat for R0
        addlw   0x03
        movwf   wtmp
        btfsc   wtmp,3
        movwf   INDF
        movf    INDF,W
        addlw   0x30
        movwf   wtmp
        btfsc   wtmp,7
        movwf   INDF
;
        goto    loop16

;output the BCD nibbles in R2,R1,R0 as ascii to async port

sendnum movf    R0,W        ;send most sig digit first
        movwf   CHAR
        call    sendchar
        movf    R1,W
        movwf   CHAR
        swapf   CHAR,1      ;first hi nibble
        call    sendchar
        swapf   CHAR,1      ;then low nibble
        call    sendchar
        movf    R2,W        ;ditto for R2 nibbles
        movwf   CHAR
        swapf   CHAR,1
        call    sendchar
        swapf   CHAR,1
        call    sendchar
prtSp  	movlw   " "         ;send a space to separate values
        goto    xmt         ;for end-of-line call prtCRNL

;convert to ascii and send bits <0:3> in CHAR

sendchar
        movf    CHAR,W      ;get the char
        andlw   0x0f        ;mask off high nibble
        addlw   "0"         ;make into ascii
        goto    xmt         ;nibble in w, send it
 
;send whatever is in W out the async serial port

prtCRNL movlw   "\r"        ;send carrage return by direct call to xmt
        call    xmt
        movlw   "\n"        ;then send newline
xmt     bsf     STATUS,RP0  ;set bank1 to access TXSTA reg
xmt1    btfss   TXSTA,TRMT  ;test for xmt reg empty     
        goto    xmt1        ;wait if necessary
        bcf     STATUS,RP0  ;back to bank 0
        movwf   TXREG       ;send it 
        return  

;*********************************************************************
;                        ASCII input routines
;*********************************************************************
;  ASCII input command formats: 

;       '!'        void current command input
;       '#'        accept current command and execute
;       'Dhh#'     accept 7 bits of switch settings plus high order enable bit
;       'Ehhhhhh#' accept 8 bits of EFC control, 16 bits of EFC setting or offset
;       'Ix#'      set single character device ID @RCL with <x>
;       'LOAD#'    disable all interrupts, jump to bootloader
;       'N0x#'     set Nmax value to <0x> (2-7)
;       'Shhhh#'   replace current PLL set point with <hhhh>

    ifdef ASCII_INPUT

;test for a known command.

readCmd call    getch       ;receive and echo the command byte
        movwf   cmd0        ;save command character to allow multiple tests
        sublw   A'!'        ;null command?
        btfsc   STATUS,Z    ;Z set if '!'
        goto    endCmd      ;yes - send crlf and exit.
;
        movf    cmd0,W      ;no - 'D' command?
        sublw   A'D'        ;
        btfsc   STATUS,Z    ;Z set if 'D'
        goto    Dcmd	    ;yes - execute 'D' command
;
        movf    cmd0,W      ;no - 'E' command?
        sublw   A'E'        ;
        btfsc   STATUS,Z    ;Z set if 'E'
        goto    Ecmd	    ;yes - execute 'E' command
;
        movf    cmd0,W      ;no - 'I' command?
        sublw   A'I'        ; @RCL
        btfsc   STATUS,Z    ;Z set if 'I'
        goto    Icmd	    ;yes - execute 'I' command
;
    ifdef loader
        movf    cmd0,W      ;no - 'LD' command?
        sublw   A'L'        ;
        btfsc   STATUS,Z    ;Z set if 'L'
        goto    LDcmd	    ;yes - test if 'LOAD' command
    endif
;
        movf    cmd0,W      ;no - 'N' command?
        sublw   A'N'        ;
        btfsc   STATUS,Z    ;Z set if 'N'
        goto    Ncmd        ;yes - execute 'N' command
;
        movf    cmd0,W      ;no - 'S' command?
        sublw   A'S'        ;
        btfsc   STATUS,Z    ;Z set if 'S'
        goto    Scmd        ;yes - execute 'S' command
        goto    badCmd      ;unrecognized command 
;
;"D" - Dip switch command

Dcmd    call    getbyte	    ;receive 8 bits (2 hex chars)
        movwf   cmd0        ;save dip alternative byte
        btfss   STATUS,Z    ;set bit 7 if user set any of 0..7
        bsf     cmd0,7
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd      ;no - user got cold feet!      
        movf    cmd0,W      ;yes - execute command... get command byte
        movwf   diptemp     ;store command byte
        goto    endCmd		
;
;"E" - EFC control and value input command

Ecmd    call    getbyte	    ;read 2 hex chars and return 8 bit value
        movwf   cmd0        ;save the 8 control bits
        call    getbyte     ;read another 2 chars	
        movwf   cmd1        ;temporarily save hi order byte of EFC set/offset value desired
        call    getbyte     ;read 2 more chars
        movwf   cmd2        ;save of lower order byte
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd      ;no - user got cold feet!      
;enter critical section - don't interrupt us until all three bytes are stored
        movf    cmd0,W      ;yes - execute command... get options byte
        bcf     INTCON,GIE  ;disable interrupts 
        movwf   EFCopts     ;store it
        movf    cmd1,W      ;get high order byte
        movwf   Hefc        ;store it
        movf    cmd2,W      ;get lo order byte
        movwf   Lefc        ;store it
        bsf     INTCON,GIE  ;re-enable interrupts
;exit critical section
        goto    endCmd 		
;
;"I" - Device ID command

Icmd    call    getch       ;receive ID character
        movwf   cmd0        ;save ID
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd 	    ;no - user got cold feet!     
        movf    cmd0,W      ;yes - execute command... get ID
        movwf   EE_Byte	    ;store in EE_Byte
        movf    devID,W     ;get previous ID	

    ifdef __16C73           ;No EEPROM in these devices
        goto    prtI1 		
    endif
    ifdef __16F73
        goto    prtI1 		
    endif

    ifdef EEPROM            ;if EEPROM enabled		
        movlw   0x11        ;set eeprom address
        call EE_Read_Byte   ;read ID back returned in W
    endif                   ;EEPROM

prtI1   call    xmt         ;print old ID 
        call    prtCRNL     ;and newline
        movf    EE_Byte,W   ;get new ID in W
        movwf   devID       ;store ID

;store ID in EEPROM 

    ifdef __16C73           ;No EEPROM in these devices
        goto    prtI2 		
    endif
    ifdef __16F73
        goto    prtI2 		
    endif
    ifdef EEPROM            ;if EEPROM enabled		
        movlw   0x11        ;yes - supply EEPROM adr to store ID
        call EE_Write_Byte  ;write the data
        clrf    devID       ;a check that we actually can w/r
        movlw   0x11        ;set eeprom address
        call EE_Read_Byte   ;read the data back returned in W
        movwf   devID       ;eeprom value to memory
    endif                   ;EEPROM

;send it out the ASCII pipe for validation

prtI2   call    xmt         ;print new ID
        goto    endCmd
;
;"LOAD" command to reload code without power cycling crystal.

LDcmd   call    getch       ;Look for "LOAD#" or "LMx" 
        movf    cmd0,W      ;no - Look for "LOAD#"
        sublw   A'O'        ;was it "O"?
        btfss   STATUS,Z
        goto    badCmd	    ;no - user got cold feet!      
        call    getch       ;get next char
        sublw   A'A'        ;was it "A"?
        btfss   STATUS,Z
        goto    badCmd	    ;no - user got cold feet!      
        call    getch       ;get next char
        sublw   A'D'        ;was it "D"?
        btfss   STATUS,Z
        goto    badCmd	    ;no - user got cold feet!      
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd	    ;no - user got cold feet!      
        clrf    INTCON      ; disable all interrupts
        clrf    STATUS      ; zero status
        goto    0x0000      ; 0x000 is a long jump to start of bootloader

;*********************************************************************
;Note - Modified by Richard McCorkle to allow AutoN to operate 
;in devices without an EEPROM or when the EEPROM is disabled

Ncmd    call    getbyte     ;receive 8 bits (2 hex chars converted to binary)
        andlw   0x07        ;value returned in W, discard bits above 0x07
        movwf   EE_Byte     ;store where the EEPROM write will find it
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd      ;no - user got cold feet!

;print previous Nmax before new value

        movf    Nmax,W      ;get previous Nmax in W

    ifdef __16C73           ;No EEPROM in these devices
        goto    prtn0 		
    endif
    ifdef __16F73
        goto    prtn0 		
    endif

    ifdef EEPROM            ;if EEPROM enabled		
        movlw   0x10        ;set eeprom address
        call EE_Read_Byte   ;read Nmax back returned in W
    endif                   ;EEPROM

prtn0   movwf   Lbyte       ;reg used to print
        clrf    Hbyte
        call    Bin2BCD     ;converts 16 bits (in Lbyte, Hbyte)
        call    sendnum     ;print it
        movf    EE_Byte,W   ;get new Nmax in W
        movwf   Nmax        ;store for later use

;store Nmax in EEPROM 

    ifdef __16C73           ;No EEPROM in these devices
        goto    prtn1 		
    endif
    ifdef __16F73
        goto    prtn1 		
    endif
    ifdef EEPROM            ;if EEPROM enabled		
        movlw   0x10        ;yes - supply EEPROM adr to store Nmax (change later?)
        call EE_Write_Byte  ;write the data
        clrf    Nmax        ;a check that we actually can w/r
        movlw   0x10        ;set eeprom address
        call EE_Read_Byte   ;read the data back returned in W
        andlw   0x07        ;value returned in W, discard possible bits above 0x07
        movwf   Nmax        ;eeprom value to memory
    endif                   ;EEPROM

;send it out the ASCII pipe for validation

prtn1   movwf   Lbyte       ;reg used to print
        clrf    Hbyte
        call    prtCRNL     ;a new line
        call    Bin2BCD     ;converts 16 bits (in Lbyte, Hbyte)
        call    sendnum     ;print it
;       call    prtCRNL     ;another newline
        goto    endCmd
;
;'S' - Setpoint write command

Scmd    call    getbyte     ;receive 8 bits (2 hex chars)
        movwf   cmd0        ;save high order set point byte
        call    getbyte     ;receive 8 more bits (2 hex chars)
        movwf   cmd1        ;save low order set point byte
        call    getch       ;now expect to receive execute permission char
        sublw   A'#'        ;was it "#"?
        btfss   STATUS,Z
        goto    badCmd      ;no - user got cold feet!
        movf    cmd0,W      ;yes - execute command... get high order byte
;enter critical section - don't interrupt us until both bytes are stored
        bcf     INTCON,GIE  ;disable interrupts
        movwf   Hsp         ;store high byte
        movf    cmd1,W
        movwf   Lsp         ;store low byte
        bsf     INTCON,GIE  ;re-enable interrupts
;exit critical section
        goto    endCmd      ;send crlf and exit.		

;*********************************************************************

badCmd  movlw   A'!'        ;send abort character
        call    xmt

endCmd  call    prtCRNL     ;send newline after command executes
        goto    bkgnd

getbyte                     ;read two hex digits and return an 8 bit value 
        call    gethex      ;get 4 MSB bits (1 hex char) into W as binary
        movwf   gbtemp      ;save
        swapf   gbtemp,1    ;move to hi nibble, leave result in gbtemp 
        call    gethex      ;get 4 LSB bits (1 hex char) into W as binary
        iorwf   gbtemp,0    ;OR together low and high nibble, save in W.
        return              ;return result in W

gethex                      ;receive and echo a HEX digit in W
        call    getch       ;receive ASCII character into W
        call    hex2bin     ;Hex to binary using W
        return

getch   movlw   D'30'       ;initilize time-out - allow 30 seconds per character
        movwf   gbTick      ;decremented each second by interrupt handler
getch0  btfsc   PIR1,RCIF   ;serial port char received?
        goto    clrgb       ;yes - go get it
        movf    gbTick,1    ;no - check time out?
        btfss   STATUS,Z    ;skip if time expired
        goto    getch0      ;continue waiting for input
        movlw   A'!'        ;timed out - simulate receive of abort character
        goto    xmt         ;xmt routine does a return

clrgb	clrf	gbTick	    ;no longer waiting for a character (bug)
getch1  movf    RCSTA,0     ;check rcv status for error
        andlw   0x06        ;select only error bits (overrun and framing errors)
        btfsc   STATUS,Z    ;error?
        call    rcverr      ;yes (reset, but garbage is echoed & returned) 
        movf    RCREG,0     ;get char from input buffer
        goto    xmt         ;echo (char returned to caller in W)

rcverr  bcf     RCSTA,CREN  ;clear overrun error by
        bsf     RCSTA,CREN  ;disable/enable rcv
        return

hex2bin movwf   h2btemp     ;convert ASCII hex char to 4 binary bits
        btfss   h2btemp,6   ;bit 6 is set? (yes for hex chars a..f and A..F)
        goto    adigit      ;no - must be a digit 0..9
        andlw   0x07        ;yes - mask off low bits, leaving 1..6
        addlw   0x09        ;bias up to 0x0a..0x0f
adigit  andlw   0x0f        ;mask low bits for 0..9 (& clr hi nibble)
        return
    
; routines used in interrupt service by EFC ascii input stuff

jam_ascii                   ;replace the computed DAC setting by the ascii value		
        movf    Hefc,W
        movwf   Hc
        movf    Lefc,W
        movwf   Mc
        clrf    Lc          ;we only input the high 16 bits of the 18-bit dac setting
        return

    endif                   ;ASCII_INPUT

;*********************************************************************
;                  subroutines to read/write EEprom
;*********************************************************************
; EE_Write_Byte - Routine to write a byte to EEprom 
; Address in W, byte MUST be in EE_Byte

    ifdef EEPROM            ;if EEprom enabled		

EE_Write_Byte
        BANK2               ;select bank2 to reach EEADR reg
        movwf   EEADR       ;eeprom address is in W
        BANK0
        movf    EE_Byte,W   ;get the data byte
        BANK2               ;back to bank2 to reach EEDATA reg
        movwf   EEDATA      ;copy the data to reg
        bsf     STATUS,RP0  ;set to bank3 to access EECON1 reg
        bcf     EECON1,EEPGD ;set to data ee ram
        bsf     EECON1,WREN ;enable writes
        DISABLE_IRQ         ;disable interrupts (irqs)
        movlw   0x55        ;required sequence !!
        movwf   EECON2
        movlw   0xaa
        movwf   EECON2
        bsf     EECON1,WR   ;begin write procedure	
        ENABLE_IRQ          ;enable irq's again
        bcf     EECON1,WREN ;disable writes ( does not affect current write cycle )
        BANK0               ;reset to bank0
;wait for the write to complete before we return
        btfss   PIR2,EEIF   ;wait for interrupt flag to be set
        goto    $ - 1
;clear interupt bit and write enable bit
        bcf     PIR2,EEIF   ;clear eewrite irq flag
        return

; EE_Read_Byte - Routine to read a byte from eeprom 
; Address in W, byte will be delivered to W

EE_Read_Byte
        BANK2               ;select bank2 to reach EEADR
        movwf   EEADR       ;eeprom address is in reg.
        bsf     STATUS,RP0  ;bank3 !!
        bcf     EECON1,EEPGD ;set to read data memory
        bsf     EECON1,RD   ;set bit to read
        bcf     STATUS,RP0  ;bank2 !!
        movf    EEDATA,W    ;move data to W
        BANK0               ;Reset to BANK0 !
        return              ;and return

    endif                   ;EEPROM

;***************************************************************
;                   Initialization Routines 
;***************************************************************
;The following initialize code is set at the end of memory to 
;save space below 0xff for code that uses computed goto's 
;Originally used org 0x700 (was 500;change 9/08) with the
;initialize code at the start of the file, but was moved here 
;so new code additions would not require changing the start 
;address unless the code length grows beyond a 2K page. 

;initialize bank 0 ports and control registers

start   clrf    PORTA       ;clear port output latches
        clrf    PORTB
        clrf    PORTC
        clrwdt         	    ;clear watch dog & prescaler before switching prescaler
        clrf    TMR0
        clrf    INTCON      ;disable all interrupts for now
        movlw   0x31        ;set sync serial port f_osc/16, rcv on rising edge
        movwf   SSPCON	    ;slowed from 0x30 which gave f_osc/4 1/20/04
	
    ifdef   WIDE_PHASE
;test for RA1 hi indicating wide phase mode selected by user
        bsf     STATUS,RP0  ;select bank 1
        movlw   0x07        ;set PORTA pin as digital (not ADC inputs)
        movwf   ADCON1
        bsf     TRISA,2	    ;set RA1 as input for a moment, we'll set as output later
        bcf     STATUS,RP0  ;back to bank 0
        bcf     Wide,0      ;first clear flag
        btfsc   PORTA,1     ;test for hi on RA1
        bsf     Wide,0      ;found it hi, set flag
    endif                   ;WIDE_PHASE

;initialize bank 1 control regs

        bsf     STATUS,RP0  ;select bank 1
        movlw   0x30        ;set PORTA pins 0,1,2,3 as outputs, 4,5 as inputs 
        movwf   TRISA       ;(during operation pin 5 is switched between input and output)
        movlw   0x07        ;set all PORTB pins as output
        movwf   TRISB       ;except interrupt RB0, VCO polarity RB1, O/C loop (Alpha) RB2
        movlw   0x07        ;set PORTA pin as digital (not ADC inputs)
        movwf   ADCON1
        movlw   0x67        ;int. on rising edge of RB0, prescaler 256 -> timer0 -> ext. ctr.
        movwf   OPTION_REG
        movlw   0xd7        ;SDI pin input (SSP mode), serial port pins, 3 user inputs
        movwf   TRISC
        clrf    PIE1        ;no int. on async xmt (tst TRMT instead)
        bsf     TXSTA,TXEN  ;enable USART xmt (async mode)
        bsf     TXSTA,BRGH  ;set USART hi speed mode
        movlw   D'38'       ;set async rate at 9600 baud (25. for 4 MHz xtal, BRGH=1)
        movwf   SPBRG       ;                            (38. for 6 MHz xtal, BRGH=1)

;back to bank 0

        bcf     STATUS,RP0  ;bank 0 for RCSTA reg
        bsf     RCSTA,CREN  ;enable USART rcv (async mode)
        bsf     RCSTA,SPEN  ;enable serial port
        bsf     INTCON,INTE ;enable interrupt on RB0
        bsf     INTCON,GIE  ;enable interrupts
        clrf    Ldata
        clrf    Hdata
        clrf    glcctr
        clrf    gbTick  
        bsf     PORTB,7     ;set DAC latch hi
        bcf     PORTB,6     ;set DAC clock line low
        bcf     PORTB,5     ;set DAC data out line lo
        bsf     FrstPss,0   ;set first pass flag
        bsf     FrstAlf,0   ;set first pass thru alpha filter flag
        bcf     UsedAlf,0   ;clear flag
        bcf     sclFlag,0   ;clear scale routine return flag
        clrf    pwm_on      ;set PWM voltage to 0
        clrf    phi
        movlw   0xFF
        movwf   phic

    ifdef   WIDE_PHASE
        btfss   Wide,0      ;test flag
        goto    p800        ;flag not set - make phase set point 800 
        movlw   0x06        ;initialize phase set point to 1600 (0x0640)
        movwf   Hsp
        movlw   0x40
        movwf   Lsp
        goto    pend
    endif                   ;WIDE_PHASE

p800    movlw   0x03        ;initialize phase set point to 800 (0x0320)
        movwf   Hsp
;       movlw   0x20        ;reduce quantization jitter: shoot for int each measurement
        movlw   0x0C        ;init sp to 780 (0x030C) integer x 30
        movwf   Lsp
pend   	movlw   D'1'        ;set IIR sample time to just use tsecs value
        movwf   minutes     ;(this is historical - allows reporting time
        movwf   toc         ;to be < sample time if desired)
        clrf    devID       ;default to no ID character @RCL

;initialize dipbits alternative to dip switch

        clrf    dipbits     ;clears switch 8 defaulting to physical switches.
        clrf    diptemp     ;clear until set by serial port command
        call    copydip     ;set dipbits from switches
        clrf    EFCopts     ;clear ASCII EFC options

;Preset Nmax for devices without an EEPROM, default is 0x00 = Off, 

;       movlw   0x05        ;AutoN on at start, Nmax = 5
        movlw   0x00        ;AutoN off at start
        movwf   Nmax

;load Nmax and ID from EEPROM if there is one and it is enabled 

    ifdef __16C73           ;No EEPROM in these devices
        goto    cNc 		
    endif
    ifdef __16F73
        goto    cNc		
    endif
    ifdef EEPROM            ;if EEPROM enabled		
        movlw   0x10        ;set eeprom address
        call EE_Read_Byte   ;read Nmax from EEPROM in W
        xorlw   0xff        ;invert the bits
        btfss   STATUS,Z    ;if 0 EEPROM not programmed
        goto    $ + 9
        clrf    EE_Byte     ;set EEPROM data to 00 
        movlw   0x10        ;address to store Nmax
        call EE_Write_Byte  ;write the data
        movlw   0x11        ;address to store devID
        call EE_Write_Byte  ;write the data
        clrf    Nmax        ;clear variables in memory
        clrf    devID
        goto    cNc         ;and proceed
        xorlw   0xff        ;else invert the bits back
        andlw   0x07        ;discard possible bits above 0x07
        movwf   Nmax        ;EEPROM value to memory
        movlw   0x11        ;set eeprom address
        call EE_Read_Byte   ;read ID from EEPROM in W
        movwf   devID       ;EEPROM value to memory
    endif                   ;EEPROM

cNc     clrf    Nctr        ;clear the advance counter

;read user input shadow and initialize accordingly

        call    setFilt     ;set IIR filter constants based on PORTC user pins
        movf    tsecs,W     ;initialize tick counter accordingly
        movwf   tick
        call    clr166      ;clear phase counter and phiwtch storage
        clrf    TMR0        ;clear internal counter
        clrf    Hic         ;clear readctr accumulator 
        clrf    Lic
        goto    bkgnd       ;jump to beginning, run background PWM code 

;***************************************************************
    de  "vsn 1.402, copyright  Brooks Shera 1997-2008"
    END 
